@@ -17,14 +17,30 @@
 #define _GNU_SOURCE
 #include <stdlib.h>
 #include <string.h>
+#include <stdarg.h>
 #include <stdio.h>
 
 #include "ordering.h"
 #include "maps.h"
 
-int is_optional(const struct policy_node *node);
-int is_tunable(const struct policy_node *node);
-int is_in_ifdef(const struct policy_node *node);
+#define SECTION_NON_ORDERED "_non_ordered"
+#define SECTION_DECLARATION "_declaration"
+
+static bool is_optional(const struct policy_node *node);
+static bool is_boolean(const struct policy_node *node);
+static bool is_tunable(const struct policy_node *node);
+static bool is_in_ifdef(const struct policy_node *node);
+
+// get next node, skip nodes marked with #selint-disable:C-001
+static const struct policy_node *ordering_next(const struct policy_node *node)
+{
+	const struct policy_node *next = node;
+	do {
+		next = dfs_next(next);
+	} while (next && next->exceptions && strstr(next->exceptions, "C-001"));
+
+	return next;
+}
 
 struct ordering_metadata *prepare_ordering_metadata(const struct check_data *data, const struct policy_node *head)
 {
@@ -38,7 +54,7 @@ struct ordering_metadata *prepare_ordering_metadata(const struct check_data *dat
 			return NULL;
 		}
 		count += 1;
-		cur = dfs_next(cur);
+		cur = ordering_next(cur);
 	}
 	calculate_average_lines(sections);
 
@@ -54,7 +70,7 @@ struct ordering_metadata *prepare_ordering_metadata(const struct check_data *dat
 
 void calculate_longest_increasing_subsequence(const struct policy_node *head,
                                               struct ordering_metadata *ordering,
-                                              enum order_difference_reason (*comp_func)(struct ordering_metadata *o,
+                                              enum order_difference_reason (*comp_func)(const struct ordering_metadata *o,
                                                                                         const struct policy_node *first,
                                                                                         const struct policy_node *second))
 {
@@ -62,12 +78,12 @@ void calculate_longest_increasing_subsequence(const struct policy_node *head,
 	int longest_seq = 0;
 	int index = 0;
 
-	struct policy_node *cur = head->next;
+	const struct policy_node *cur = head->next;
 
 	while (cur) {
 		// Save the node in the array
 		if (cur->flavor == NODE_START_BLOCK) {
-			cur = dfs_next(cur);
+			cur = ordering_next(cur);
 			continue;
 		}
 		nodes[index].node = cur;
@@ -97,7 +113,7 @@ void calculate_longest_increasing_subsequence(const struct policy_node *head,
 			longest_seq = low;
 		}
 		index++;
-		cur = dfs_next(cur);
+		cur = ordering_next(cur);
 	}
 
 	// Mark LIS elements
@@ -159,10 +175,12 @@ const char *get_section(const struct policy_node *node)
 	case NODE_IF_FILE:
 	case NODE_FC_FILE:
 	case NODE_SPT_FILE:
+	case NODE_AV_FILE:
 		return NULL; // Should never happen
 	case NODE_HEADER:
-		return "_non_ordered"; // Guaranteed at top by grammar
+		return SECTION_NON_ORDERED; // Guaranteed at top by grammar
 	case NODE_AV_RULE:
+	case NODE_XAV_RULE:
 		if (node->data.av_data->flavor == AV_RULE_NEVERALLOW) {
 			// These are somewhat of a unique situation, and the style guide
 			// doesn't mention them explicitly.  Maybe they should just group
@@ -170,15 +188,15 @@ const char *get_section(const struct policy_node *node)
 			// Additionally, the below code assumes that the first string in
 			// the sources is a type or attribute, but in the case of neverallows
 			// it can be "~"
-			return "_non_ordered";
+			return SECTION_NON_ORDERED;
 		}
 		if (node->data.av_data->flavor == AV_RULE_AUDITALLOW) {
-			return "_non_ordered";
+			return SECTION_NON_ORDERED;
 		}
 		if (node->data.av_data->perms &&
 		    (str_in_sl("associate", node->data.av_data->perms) ||
 		     str_in_sl("mounton", node->data.av_data->perms))) {
-			return "_non_ordered"; // Can be transform or with rules
+			return SECTION_NON_ORDERED; // Can be transform or with rules
 		}
 		// The case of multiple source types is weird.  For now
 		// just using the first one seems fine.
@@ -190,57 +208,60 @@ const char *get_section(const struct policy_node *node)
 		// just using the first one seems fine.
 		return node->data.av_data->sources->string;
 	case NODE_RT_RULE:
-		return "_non_ordered";
+		return SECTION_NON_ORDERED;
 	case NODE_ROLE_ALLOW:
 	case NODE_ROLE_TYPES:
 		// These are not in the style guide. I normally see them grouped
 		// with declarations, but maybe a future ordering configuration
 		// can sort them that way
-		return "_non_ordered";
+		return SECTION_NON_ORDERED;
 	case NODE_DECL:
 	case NODE_ALIAS:
 	case NODE_TYPE_ALIAS:
 	case NODE_TYPE_ATTRIBUTE:
 	case NODE_ROLE_ATTRIBUTE:
 		if (is_in_require(node)) {
-			return "_non_ordered";
+			return SECTION_NON_ORDERED;
 		} else {
-			return "_declarations";
+			return SECTION_DECLARATION;
 		}
 	case NODE_M4_CALL:
-		return "_non_ordered"; // TODO: It's probably way more
-	// complicated than this
+	case NODE_M4_SIMPLE_MACRO:
+		return SECTION_NON_ORDERED; // TODO: It's probably way more complicated than this
 	case NODE_OPTIONAL_POLICY:
 	case NODE_OPTIONAL_ELSE:
+	case NODE_BOOLEAN_POLICY:
 	case NODE_TUNABLE_POLICY:
 	case NODE_IFDEF:
+	case NODE_IFELSE:
 		return get_section(node->first_child);
 	case NODE_M4_ARG:
-		return "_non_ordered"; //TODO
+		return SECTION_NON_ORDERED; //TODO
 	case NODE_START_BLOCK:
 		if (node->next) {
 			return get_section(node->next);
 		} else {
-			return "_non_ordered"; // empty block
+			return SECTION_NON_ORDERED; // empty block
 		}
 	case NODE_IF_CALL:
 		// check for filetrans_if first to treat interfaces with the
 		// flags filetrans and transform as _non-ordered
 		if (is_filetrans_if(node->data.ic_data->name)) {
-			return "_non_ordered";
+			return SECTION_NON_ORDERED;
 		} else if (!is_optional(node) &&
 		    !is_in_ifdef(node) &&
+		    !is_boolean(node) &&
 		    !is_tunable(node) &&
 		    (look_up_in_template_map(node->data.ic_data->name) ||
 		     is_transform_if(node->data.ic_data->name) ||
 		     is_role_if(node->data.ic_data->name))) {
-			return "_declarations";
+			return SECTION_DECLARATION;
 		} else {
 			if (node->data.ic_data->args) {
 				return node->data.ic_data->args->string;
 			} else {
 				// Empty interface call
-				return "_non_ordered";
+				return SECTION_NON_ORDERED;
 			}
 		}
 	case NODE_TEMP_DEF:
@@ -248,16 +269,16 @@ const char *get_section(const struct policy_node *node)
 		return NULL;           // if files only
 	case NODE_REQUIRE:
 	case NODE_GEN_REQ:
-		return "_non_ordered"; // Not in style guide
+		return SECTION_NON_ORDERED; // Not in style guide
 	case NODE_PERMISSIVE:
-		return "_non_ordered"; // Not in style guide
+		return SECTION_NON_ORDERED; // Not in style guide
 	case NODE_FC_ENTRY:
 		return NULL;           // fc files only
 	case NODE_COMMENT:
 	case NODE_EMPTY:
 	case NODE_SEMICOLON:
 	case NODE_ERROR:
-		return "_non_ordered";
+		return SECTION_NON_ORDERED;
 	default:
 		// Should never happen
 		return NULL;
@@ -272,7 +293,7 @@ void calculate_average_lines(struct section_data *sections)
 	}
 }
 
-float get_avg_line_by_name(const char *section_name, struct section_data *sections)
+float get_avg_line_by_name(const char *section_name, const struct section_data *sections)
 {
 	while (0 != strcmp(sections->section_name, section_name)) {
 		sections = sections->next;
@@ -283,25 +304,26 @@ float get_avg_line_by_name(const char *section_name, struct section_data *sectio
 	return sections->avg_line;
 }
 
-static int is_self_rule(const struct policy_node *node)
+static bool is_self_rule(const struct policy_node *node)
 {
-	return node->flavor == NODE_AV_RULE &&
+	return (node->flavor == NODE_AV_RULE || node->flavor == NODE_XAV_RULE) &&
 	       node->data.av_data &&
 	       node->data.av_data->targets &&
 	       0 == strcmp(node->data.av_data->targets->string, "self");
 }
 
-static int is_own_module_rule(const struct policy_node *node, const char *current_mod_name)
+static bool is_own_module_rule(const struct policy_node *node, const char *current_mod_name)
 {
 	if (node->flavor != NODE_AV_RULE &&
+	    node->flavor != NODE_XAV_RULE &&
 	    node->flavor != NODE_IF_CALL) {
-		return 0;
+		return false;
 	}
 
 	if (node->flavor == NODE_IF_CALL) {
 		// These should actually be patterns, not real calls
 		if (look_up_in_ifs_map(node->data.ic_data->name)) {
-			return 0;
+			return false;
 		}
 	}
 	struct string_list *names = get_names_in_node(node);
@@ -314,7 +336,7 @@ static int is_own_module_rule(const struct policy_node *node, const char *curren
 		if (module_of_type_or_attr &&
 		    0 != strcmp(module_of_type_or_attr, current_mod_name)) {
 			free_string_list(names);
-			return 0;
+			return false;
 		}
 		cur = cur->next;
 	}
@@ -324,110 +346,169 @@ static int is_own_module_rule(const struct policy_node *node, const char *curren
 	// "read_file_perms" for example.  However, in normal mode without context
 	// this could definitely be a problem because we won't find types from
 	// other modules
-	return 1;
+	return true;
 }
 
-static int is_kernel_mod_if_call(const struct policy_node *node)
+static bool is_kernel_mod_if_call(const struct policy_node *node)
 {
 	if (node->flavor != NODE_IF_CALL) {
-		return 0;
+		return false;
 	}
 	const char *mod_name = look_up_in_ifs_map(node->data.ic_data->name);
 	if (!mod_name) {
-		return 0;
+		return false;
 	}
 	if (0 == strcmp("kernel", mod_name)) {
-		return 1;
+		return true;
 	}
-	return 0;
+	return false;
 }
 
-static int is_own_mod_if_call(const struct policy_node *node, const char *current_mod_name)
+static bool is_own_mod_if_call(const struct policy_node *node, const char *current_mod_name)
 {
 	if (node->flavor != NODE_IF_CALL) {
-		return 0;
+		return false;
 	}
 	const char *mod_name = look_up_in_ifs_map(node->data.ic_data->name);
 	if (!mod_name) {
-		return 0;
+		return false;
 	}
 
 	if (current_mod_name &&
 	    0 != strcmp(current_mod_name, mod_name)) {
-		return 0;
+		return false;
 	}
-	return 1;
+	return true;
 }
 
-static int check_call_layer(const struct policy_node *node, const char *layer_to_check)
+static bool is_related_mod_if_call(const struct policy_node *node, const char *current_mod_name)
 {
 	if (node->flavor != NODE_IF_CALL) {
-		return 0;
+		return false;
+	}
+
+	const char *current_mod_prefix = strchr(current_mod_name, '_');
+	size_t cur_prefix_len;
+	if (current_mod_prefix) {
+		cur_prefix_len = (size_t)(current_mod_prefix - current_mod_name);
+	} else {
+		cur_prefix_len = strlen(current_mod_name);
+	}
+
+	const char *mod_name = look_up_in_ifs_map(node->data.ic_data->name);
+	if (!mod_name) {
+		return false;
+	}
+
+	const char *mod_prefix = strchr(mod_name, '_');
+	size_t prefix_len;
+	if (mod_prefix) {
+		prefix_len = (size_t)(mod_prefix - mod_name);
+	} else {
+		prefix_len = strlen(mod_name);
+	}
+
+	if (cur_prefix_len == prefix_len &&
+	    0 == strncmp(current_mod_name, mod_name, cur_prefix_len)) {
+		return true;
+	}
+
+	return false;
+}
+
+static bool check_call_layer(const struct policy_node *node, const char *layer_to_check)
+{
+	if (node->flavor != NODE_IF_CALL) {
+		return false;
 	}
 	const char *mod_name = look_up_in_ifs_map(node->data.ic_data->name);
 	if (!mod_name) {
 		// not an actual interface
-		return 0;
+		return false;
 	}
 	const char *layer_name = look_up_in_mod_layers_map(mod_name);
 	if (!layer_name) {
-		return 0;
+		return false;
 	}
 	return (0 == strcmp(layer_name, layer_to_check));
 }
 
-static int is_kernel_layer_if_call(const struct policy_node *node)
+static bool is_kernel_layer_if_call(const struct policy_node *node)
 {
 	return check_call_layer(node, "kernel");
 }
 
-static int is_system_layer_if_call(const struct policy_node *node)
+static bool is_system_layer_if_call(const struct policy_node *node)
 {
 	return check_call_layer(node, "system");
 }
 
-int is_optional(const struct policy_node *node)
+static bool is_optional(const struct policy_node *node)
 {
-	int ret = 0;
+	bool ret = false;
 	while (node) {
 		if (node->flavor == NODE_OPTIONAL_POLICY ||
 		    node->flavor == NODE_OPTIONAL_ELSE) {
-			ret = 1;
-		} else if (node->flavor == NODE_TUNABLE_POLICY ||
-		           node->flavor == NODE_IFDEF) {
-			ret = 0;
+			ret = true;
+		} else if (node->flavor == NODE_BOOLEAN_POLICY ||
+		           node->flavor == NODE_TUNABLE_POLICY ||
+		           node->flavor == NODE_IFDEF ||
+		           node->flavor == NODE_IFELSE) {
+			ret = false;
 		}
 		node = node->parent;
 	}
 	return ret;
 }
 
-int is_tunable(const struct policy_node *node)
+static bool is_boolean(const struct policy_node *node)
 {
-	int ret = 0;
+	bool ret = false;
+	while (node) {
+		if (node->flavor == NODE_BOOLEAN_POLICY) {
+			ret = true;
+		} else if (node->flavor == NODE_TUNABLE_POLICY ||
+		           node->flavor == NODE_OPTIONAL_POLICY ||
+		           node->flavor == NODE_OPTIONAL_ELSE ||
+		           node->flavor == NODE_IFDEF ||
+		           node->flavor == NODE_IFELSE) {
+			ret = false;
+		}
+		node = node->parent;
+	}
+	return ret;
+}
+
+static bool is_tunable(const struct policy_node *node)
+{
+	bool ret = false;
 	while (node) {
 		if (node->flavor == NODE_TUNABLE_POLICY) {
-			ret = 1;
-		} else if (node->flavor == NODE_OPTIONAL_POLICY ||
+			ret = true;
+		} else if (node->flavor == NODE_BOOLEAN_POLICY ||
+		           node->flavor == NODE_OPTIONAL_POLICY ||
 		           node->flavor == NODE_OPTIONAL_ELSE ||
-		           node->flavor == NODE_IFDEF) {
-			ret = 0;
+		           node->flavor == NODE_IFDEF ||
+		           node->flavor == NODE_IFELSE) {
+			ret = false;
 		}
 		node = node->parent;
 	}
 	return ret;
 }
 
-int is_in_ifdef(const struct policy_node *node)
+static bool is_in_ifdef(const struct policy_node *node)
 {
-	int ret = 0;
+	bool ret = false;
 	while (node) {
-		if (node->flavor == NODE_IFDEF) {
-			ret = 1;
+		if (node->flavor == NODE_IFDEF ||
+		    node->flavor == NODE_IFELSE) {
+			ret = true;
 		} else if (node->flavor == NODE_OPTIONAL_POLICY ||
 		           node->flavor == NODE_OPTIONAL_ELSE ||
+		           node->flavor == NODE_BOOLEAN_POLICY ||
 		           node->flavor == NODE_TUNABLE_POLICY) {
-			ret = 0;
+			ret = false;
 		}
 
 		node = node->parent;
@@ -435,7 +516,9 @@ int is_in_ifdef(const struct policy_node *node)
 	return ret;
 }
 
-enum local_subsection get_local_subsection(const char *mod_name, const struct policy_node *node)
+enum local_subsection get_local_subsection(const char *mod_name,
+                                           const struct policy_node *node,
+                                           enum order_conf variant)
 {
 	if (!node) {
 		return LSS_UNKNOWN;
@@ -444,6 +527,8 @@ enum local_subsection get_local_subsection(const char *mod_name, const struct po
 		return LSS_BUILD_OPTION;
 	} else if (is_optional(node)) {
 		return LSS_OPTIONAL;
+	} else if (is_boolean(node)) {
+		return LSS_BOOLEAN;
 	} else if (is_tunable(node)) {
 		return LSS_TUNABLE;
 	} else if (is_self_rule(node)) {
@@ -452,11 +537,13 @@ enum local_subsection get_local_subsection(const char *mod_name, const struct po
 		return LSS_OWN;
 	} else if (is_own_mod_if_call(node, mod_name)) {
 		return LSS_OWN;
+	} else if (variant != ORDER_REF && is_related_mod_if_call(node, mod_name)) {
+		return LSS_RELATED;
 	} else if (is_kernel_mod_if_call(node)) {
 		return LSS_KERNEL_MOD;
-	} else if (is_kernel_layer_if_call(node)) {
+	} else if (variant != ORDER_LIGHT && is_kernel_layer_if_call(node)) {
 		return LSS_KERNEL;
-	} else if (is_system_layer_if_call(node)) {
+	} else if (variant != ORDER_LIGHT && is_system_layer_if_call(node)) {
 		return LSS_SYSTEM;
 	} else if (node->flavor == NODE_IF_CALL) {
 		return LSS_OTHER;
@@ -472,7 +559,7 @@ enum local_subsection get_local_subsection(const char *mod_name, const struct po
  *       foo   and foo_t
  *       foo   and foo_r
  */
-static int is_same_section(const char *first_section_name, const char *second_section_name)
+static bool is_same_section(const char *first_section_name, const char *second_section_name)
 {
 	size_t first_length = strlen(first_section_name);
 	size_t second_length = strlen(second_section_name);
@@ -504,10 +591,10 @@ static int is_same_section(const char *first_section_name, const char *second_se
 #define CHECK_FLAVOR_ORDERING(data_flavor, comp, ret) \
 	CHECK_ORDERING(first->data.data_flavor->flavor, second->data.data_flavor->flavor, comp, ret)
 
-enum order_difference_reason compare_nodes_refpolicy_generic(struct ordering_metadata *ordering_data,
+enum order_difference_reason compare_nodes_refpolicy_generic(const struct ordering_metadata *ordering_data,
                                                              const struct policy_node *first,
                                                              const struct policy_node *second,
-						             enum order_conf variant)
+                                                             enum order_conf variant)
 {
 	const char *first_section_name = get_section(first);
 	const char *second_section_name = get_section(second);
@@ -516,14 +603,14 @@ enum order_difference_reason compare_nodes_refpolicy_generic(struct ordering_met
 		return ORDERING_ERROR;
 	}
 
-	if (0 == strcmp(first_section_name, "_non_ordered") ||
-	    0 == strcmp(second_section_name, "_non_ordered")) {
+	if (0 == strcmp(first_section_name, SECTION_NON_ORDERED) ||
+	    0 == strcmp(second_section_name, SECTION_NON_ORDERED)) {
 		return ORDER_EQUAL;
 	}
 
 	if (!is_same_section(first_section_name, second_section_name)) {
-		if (0 != strcmp(first_section_name, "_declarations") &&
-		    (0 == strcmp(second_section_name, "_declarations") ||
+		if (0 != strcmp(first_section_name, SECTION_DECLARATION) &&
+		    (0 == strcmp(second_section_name, SECTION_DECLARATION) ||
 		     (get_avg_line_by_name(first_section_name, ordering_data->sections) >
 		      get_avg_line_by_name(second_section_name, ordering_data->sections))) &&
 		    // allow raw section alphabetically following another raw section
@@ -536,7 +623,7 @@ enum order_difference_reason compare_nodes_refpolicy_generic(struct ordering_met
 
 	// If we made it to this point the two nodes are in the same section
 
-	if (0 == strcmp(first_section_name, "_declarations")) {
+	if (0 == strcmp(first_section_name, SECTION_DECLARATION)) {
 		if (first->flavor == NODE_DECL && second->flavor == NODE_DECL) {
 			if (first->data.d_data->flavor != second->data.d_data->flavor) {
 				CHECK_FLAVOR_ORDERING(d_data, DECL_BOOL, ORDER_DECLARATION_SUBSECTION);
@@ -550,40 +637,91 @@ enum order_difference_reason compare_nodes_refpolicy_generic(struct ordering_met
 	}
 
 	// Local policy rules sections
-	enum local_subsection lss_first = get_local_subsection(ordering_data->mod_name, first);
-	enum local_subsection lss_second = get_local_subsection(ordering_data->mod_name, second);
+	const enum local_subsection lss_first = get_local_subsection(ordering_data->mod_name, first, variant);
+	const enum local_subsection lss_second = get_local_subsection(ordering_data->mod_name, second, variant);
 
 	if (lss_first == LSS_UNKNOWN || lss_second == LSS_UNKNOWN) {
 		return ORDER_EQUAL; // ... Maybe? Should this case be handled earlier?
 	}
 
-	CHECK_ORDERING(lss_first, lss_second, LSS_SELF, ORDER_LOCAL_SUBSECTION);
-	CHECK_ORDERING(lss_first, lss_second, LSS_OWN, ORDER_LOCAL_SUBSECTION);
+	if (lss_first != lss_second) {
+		CHECK_ORDERING(lss_first, lss_second, LSS_SELF, ORDER_LOCAL_SUBSECTION);
+		CHECK_ORDERING(lss_first, lss_second, LSS_OWN, ORDER_LOCAL_SUBSECTION);
 
-	if (variant == ORDER_REF) {
-		CHECK_ORDERING(lss_first, lss_second, LSS_KERNEL_MOD, ORDER_LOCAL_SUBSECTION);
-		CHECK_ORDERING(lss_first, lss_second, LSS_KERNEL, ORDER_LOCAL_SUBSECTION);
-		CHECK_ORDERING(lss_first, lss_second, LSS_SYSTEM, ORDER_LOCAL_SUBSECTION);
-		CHECK_ORDERING(lss_first, lss_second, LSS_OTHER, ORDER_LOCAL_SUBSECTION);
-		CHECK_ORDERING(lss_first, lss_second, LSS_BUILD_OPTION, ORDER_LOCAL_SUBSECTION);
-		CHECK_ORDERING(lss_first, lss_second, LSS_CONDITIONAL, ORDER_LOCAL_SUBSECTION);
-		CHECK_ORDERING(lss_first, lss_second, LSS_TUNABLE, ORDER_LOCAL_SUBSECTION);
-		CHECK_ORDERING(lss_first, lss_second, LSS_OPTIONAL, ORDER_LOCAL_SUBSECTION);
+		if (variant == ORDER_REF) {
+			CHECK_ORDERING(lss_first, lss_second, LSS_KERNEL_MOD, ORDER_LOCAL_SUBSECTION);
+			CHECK_ORDERING(lss_first, lss_second, LSS_KERNEL, ORDER_LOCAL_SUBSECTION);
+			CHECK_ORDERING(lss_first, lss_second, LSS_SYSTEM, ORDER_LOCAL_SUBSECTION);
+			CHECK_ORDERING(lss_first, lss_second, LSS_OTHER, ORDER_LOCAL_SUBSECTION);
+			CHECK_ORDERING(lss_first, lss_second, LSS_BUILD_OPTION, ORDER_LOCAL_SUBSECTION);
+			CHECK_ORDERING(lss_first, lss_second, LSS_BOOLEAN, ORDER_LOCAL_SUBSECTION);
+			CHECK_ORDERING(lss_first, lss_second, LSS_TUNABLE, ORDER_LOCAL_SUBSECTION);
+			CHECK_ORDERING(lss_first, lss_second, LSS_OPTIONAL, ORDER_LOCAL_SUBSECTION);
+		} else if (variant == ORDER_LIGHT) {
+			CHECK_ORDERING(lss_first, lss_second, LSS_RELATED, ORDER_LOCAL_SUBSECTION);
+			CHECK_ORDERING(lss_first, lss_second, LSS_KERNEL_MOD, ORDER_LOCAL_SUBSECTION);
+			CHECK_ORDERING(lss_first, lss_second, LSS_OTHER, ORDER_LOCAL_SUBSECTION);
+			CHECK_ORDERING(lss_first, lss_second, LSS_BUILD_OPTION, ORDER_LOCAL_SUBSECTION);
+			CHECK_ORDERING(lss_first, lss_second, LSS_BOOLEAN, ORDER_LOCAL_SUBSECTION);
+			CHECK_ORDERING(lss_first, lss_second, LSS_TUNABLE, ORDER_LOCAL_SUBSECTION);
+			CHECK_ORDERING(lss_first, lss_second, LSS_OPTIONAL, ORDER_LOCAL_SUBSECTION);
+		}
 	}
 
-	// TODO: alphabetical
+	if (variant == ORDER_REF || variant == ORDER_LIGHT) {
+		// alphabetical top-level interfaces
+		if (lss_first != LSS_BUILD_OPTION &&
+		    lss_first != LSS_BOOLEAN &&
+		    lss_first != LSS_TUNABLE &&
+		    (lss_first != LSS_OPTIONAL ||
+		    // sort optionals only based on first node
+		    (lss_first == lss_second && first->prev->flavor == NODE_START_BLOCK && second->prev->flavor == NODE_START_BLOCK)) &&
+		    first->flavor == NODE_IF_CALL &&
+		    second->flavor == NODE_IF_CALL) {
+			const char *first_mod_name = look_up_in_ifs_map(first->data.ic_data->name);
+			const char *second_mod_name = look_up_in_ifs_map(second->data.ic_data->name);
+
+			if (first_mod_name && second_mod_name) {
+				const int compare = strcmp(first_mod_name, second_mod_name);
+				if (compare < 0) {
+					return ORDER_ALPHABETICAL;
+				} else if (compare > 0) {
+					return -ORDER_ALPHABETICAL;
+				}
+			}
+		}
+
+		// alphabetical optionals
+		// sort by the first node in a optional block
+		if (lss_first == LSS_OPTIONAL &&
+		    lss_second == LSS_OPTIONAL &&
+		    first->first_child && first->first_child->next &&
+		    second->first_child && second->first_child->next) {
+			return compare_nodes_refpolicy_generic(ordering_data,
+							    first->first_child->next,
+							    second->first_child->next,
+							    variant);
+		}
+	}
 
 	return ORDER_EQUAL;
 }
 
-enum order_difference_reason compare_nodes_refpolicy(struct ordering_metadata *ordering_data,
+enum order_difference_reason compare_nodes_refpolicy(const struct ordering_metadata *ordering_data,
                                                      const struct policy_node *first,
                                                      const struct policy_node *second)
 {
 	return compare_nodes_refpolicy_generic(ordering_data, first, second, ORDER_REF);
 }
 
-enum order_difference_reason compare_nodes_refpolicy_lax(struct ordering_metadata *ordering_data,
+enum order_difference_reason compare_nodes_refpolicy_light(const struct ordering_metadata *ordering_data,
+                                                           const struct policy_node *first,
+                                                           const struct policy_node *second)
+{
+	return compare_nodes_refpolicy_generic(ordering_data, first, second, ORDER_LIGHT);
+}
+
+enum order_difference_reason compare_nodes_refpolicy_lax(const struct ordering_metadata *ordering_data,
                                                          const struct policy_node *first,
                                                          const struct policy_node *second)
 {
@@ -594,32 +732,34 @@ const char *lss_to_string(enum local_subsection lss)
 {
 	switch (lss) {
 	case LSS_SELF:
-		return "self";
+		return "self rule";
 	case LSS_OWN:
-		return "own module rules";
+		return "own module rule";
+	case LSS_RELATED:
+		return "related module interface";
 	case LSS_KERNEL_MOD:
-		return "kernel_mod";
+		return "kernel module interface";
 	case LSS_KERNEL:
-		return "kernel";
+		return "kernel layer interface";
 	case LSS_SYSTEM:
-		return "system";
+		return "system layer interface";
 	case LSS_OTHER:
-		return "general interfaces";
+		return "general interface";
 	case LSS_BUILD_OPTION:
-		return "build options";
-	case LSS_CONDITIONAL:
-		return "conditional blocks";
+		return "build option";
+	case LSS_BOOLEAN:
+		return "boolean policy block";
 	case LSS_TUNABLE:
-		return "tunable policy blocks";
+		return "tunable policy block";
 	case LSS_OPTIONAL:
-		return "optional policy blocks";
+		return "optional policy block";
 	case LSS_UNKNOWN:
 	default:
 		return "unknown subsection";
 	}
 }
 
-char *get_ordering_reason(struct ordering_metadata *order_data, unsigned int index)
+char *get_ordering_reason(struct ordering_metadata *order_data, unsigned int index, enum order_conf variant)
 {
 	unsigned int distance = 1;
 	unsigned int nearest_index = 0;
@@ -627,9 +767,10 @@ char *get_ordering_reason(struct ordering_metadata *order_data, unsigned int ind
 	while (nearest_index == 0) {
 		if (distance < index &&
 		    order_data->nodes[index-distance].in_order) {
-			reason = compare_nodes_refpolicy(order_data,
-							 order_data->nodes[index-distance].node,
-							 order_data->nodes[index].node);
+			reason = compare_nodes_refpolicy_generic(order_data,
+								 order_data->nodes[index-distance].node,
+								 order_data->nodes[index].node,
+								 variant);
 			if (reason < 0) {
 				nearest_index = index - distance;
 				break;
@@ -637,9 +778,10 @@ char *get_ordering_reason(struct ordering_metadata *order_data, unsigned int ind
 		}
 		if (index + distance < order_data->order_node_len &&
 		    order_data->nodes[index+distance].in_order) {
-			reason = compare_nodes_refpolicy(order_data,
-	                                                 order_data->nodes[index].node,
-	                                                 order_data->nodes[index+distance].node);
+			reason = compare_nodes_refpolicy_generic(order_data,
+								 order_data->nodes[index].node,
+								 order_data->nodes[index+distance].node,
+								 variant);
 			if (reason < 0) {
 				nearest_index = index + distance;
 				break;
@@ -655,18 +797,18 @@ char *get_ordering_reason(struct ordering_metadata *order_data, unsigned int ind
 	const struct policy_node *this_node = order_data->nodes[index].node;
 	const struct policy_node *other_node = order_data->nodes[nearest_index].node;
 
-	const char *before_after = NULL;
+	const char *before_after;
 	if (nearest_index > index) {
 		before_after = "before";
 	} else {
 		before_after = "after";
 	}
 
-	const char *reason_str = NULL;
+	const char *reason_str;
 	char *followup_str = NULL;
 	enum local_subsection other_lss;
-	const char *node_section = NULL;
-	const char *other_section = NULL;
+	const char *node_section;
+	const char *other_section;
 
 	switch (-reason) {
 	case ORDER_EQUAL:
@@ -677,10 +819,10 @@ char *get_ordering_reason(struct ordering_metadata *order_data, unsigned int ind
 		if (!node_section || !other_section) {
 			return NULL; // Error
 		}
-		if (0 == strcmp("_declarations", node_section)) {
+		if (0 == strcmp(SECTION_DECLARATION, node_section)) {
 			// This is the first section
 			reason_str = "that is not a declaration";
-		} else if (0 == strcmp("_declarations", other_section)) {
+		} else if (0 == strcmp(SECTION_DECLARATION, other_section)) {
 			// The other section is the first section
 			if (other_node->flavor == NODE_IF_CALL && is_transform_if(other_node->data.ic_data->name)) {
 				reason_str = "that is a transform interface";
@@ -689,21 +831,18 @@ char *get_ordering_reason(struct ordering_metadata *order_data, unsigned int ind
 			}
 		} else {
 			reason_str = "that is in a different section";
-			int r = asprintf(&followup_str, "  (This node is in the section for %s rules and the other is in the section for %s rules.)", node_section, other_section);
-			if (r == -1) {
-				return NULL; //ERROR
-			}
+		}
+
+		int r = asprintf(&followup_str, "  (This node is in the section for %s rules and the other is in the section for %s rules.)", node_section, other_section);
+		if (r == -1) {
+			return NULL; //ERROR
 		}
 		break;
 	case ORDER_DECLARATION_SUBSECTION:
 		reason_str = "that is associated with a different sort of declaration";
 		break;
-	case ORDER_LAYERS:
-		// TODO
-		reason_str = "that is in another layer";
-		break;
 	case ORDER_LOCAL_SUBSECTION:
-		other_lss = get_local_subsection(order_data->mod_name, other_node);
+		other_lss = get_local_subsection(order_data->mod_name, other_node, variant);
 		switch (other_lss) {
 		case LSS_SELF:
 			reason_str = "that is a self rule";
@@ -711,6 +850,9 @@ char *get_ordering_reason(struct ordering_metadata *order_data, unsigned int ind
 		case LSS_OWN:
 			reason_str = "that refers to types owned by this module";
 			break;
+		case LSS_RELATED:
+			reason_str = "that calls an interface located in a related module";
+			break;
 		case LSS_KERNEL_MOD:
 			reason_str = "that calls an interface located in the kernel module";
 			break;
@@ -721,13 +863,14 @@ char *get_ordering_reason(struct ordering_metadata *order_data, unsigned int ind
 			reason_str = "that calls an interface located in the system layer";
 			break;
 		case LSS_OTHER:
-			reason_str = "that calls an interface not located in the kernel or system layer";
+			reason_str = variant == ORDER_REF ? "that calls an interface not located in the kernel or system layer"
+							  : "that calls a non-optional interface";
 			break;
 		case LSS_BUILD_OPTION:
 			reason_str = "that is controlled by a build option";
 			break;
-		case LSS_CONDITIONAL:
-			reason_str = "that is in a conditional policy block";
+		case LSS_BOOLEAN:
+			reason_str = "that is in a boolean block";
 			break;
 		case LSS_TUNABLE:
 			reason_str = "that is in a tunable block";
@@ -742,14 +885,13 @@ char *get_ordering_reason(struct ordering_metadata *order_data, unsigned int ind
 			return NULL;
 		}
 		if (other_lss == LSS_KERNEL || other_lss == LSS_SYSTEM || other_lss == LSS_OTHER) {
-			enum local_subsection this_lss = get_local_subsection(order_data->mod_name, this_node);
-			if (this_lss == LSS_KERNEL || this_lss == LSS_SYSTEM) {
-				int r = asprintf(&followup_str, "  (This interface is in the %s layer.)", lss_to_string(this_lss));
-				if (r == -1) {
-					return NULL; //ERROR
-				}
+			enum local_subsection this_lss = get_local_subsection(order_data->mod_name, this_node, variant);
+			if (this_lss == LSS_KERNEL) {
+				followup_str = strdup("  (This interface is in the kernel layer.)");
+			} else if (this_lss == LSS_SYSTEM) {
+				followup_str = strdup("  (This interface is in the system layer.)");
 			} else if (this_lss == LSS_OTHER) {
-				followup_str = strdup("  (This interface is in a layer other than kernel or system)");
+				followup_str = strdup("  (This interface is in a layer other than kernel or system.)");
 			} else if (this_lss == LSS_KERNEL_MOD) {
 				followup_str = strdup("  (This interface is in the kernel module.)");
 			}
@@ -772,7 +914,7 @@ char *get_ordering_reason(struct ordering_metadata *order_data, unsigned int ind
 	size_t str_len = strlen(reason_str) +
 	                 strlen(before_after) +
 	                 strlen("Line out of order.  It is of type ") +
-	                 strlen(lss_to_string(get_local_subsection(order_data->mod_name, this_node))) + 1 +
+	                 strlen(lss_to_string(get_local_subsection(order_data->mod_name, this_node, variant))) + 1 +
 	                 strlen(" line ") +
 	                 13; // 13 is enough for the maximum
 	                     // length of an unsigned int (10)
@@ -786,13 +928,14 @@ char *get_ordering_reason(struct ordering_metadata *order_data, unsigned int ind
 
 	ssize_t written = snprintf(ret, str_len,
 	                           "Line out of order.  It is of type %s %s line %u %s.",
-	                           lss_to_string(get_local_subsection(order_data->mod_name, this_node)),
+	                           lss_to_string(get_local_subsection(order_data->mod_name, this_node, variant)),
 	                           before_after,
 	                           other_node->lineno,
 	                           reason_str);
 
 	if (written < 0) {
 		free(followup_str);
+		free(ret);
 		return NULL;
 	}
 
@@ -804,31 +947,6 @@ char *get_ordering_reason(struct ordering_metadata *order_data, unsigned int ind
 	return ret;
 }
 
-int check_transform_interface_suffix(char *if_name)
-{
-	char *suffix = strrchr(if_name, '_');
-	if (suffix &&
-	    (0 == strcmp(suffix, "_type") ||
-	     0 == strcmp(suffix, "_file") ||
-	     0 == strcmp(suffix, "_domain") ||
-	     0 == strcmp(suffix, "_node") ||
-	     // Next three are found in mta module
-	     0 == strcmp(suffix, "_agent") ||
-	     0 == strcmp(suffix, "_delivery") ||
-	     0 == strcmp(suffix, "_sender") ||
-	     0 == strcmp(suffix, "_boolean") ||
-	     0 == strcmp(suffix, "_content") ||
-	     0 == strcmp(suffix, "_constrained") ||
-	     0 == strcmp(suffix, "_executable") ||
-	     0 == strcmp(suffix, "_exemption") ||
-	     0 == strcmp(suffix, "_object") ||
-	     0 == strcmp(suffix, "_mountpoint"))) {
-		return 1;
-	} else {
-		return 0;
-	}
-}
-
 void free_ordering_metadata(struct ordering_metadata *to_free)
 {
 	if (to_free == NULL) {
