@@ -21,11 +21,9 @@
 #include "selint_error.h"
 #include "tree.h"
 #include "template.h"
-#include "ordering.h"
+#include "util.h"
 #include "perm_macro.h"
 
-extern int yylex_destroy(void);
-
 char *module_name = NULL;
 
 enum selint_error insert_header(struct policy_node **cur, const char *mn,
@@ -241,13 +239,51 @@ enum selint_error insert_av_rule(struct policy_node **cur,
 	return SELINT_SUCCESS;
 }
 
-enum selint_error insert_role_allow(struct policy_node **cur, const char *from_role,
-                                    const char *to_role, unsigned int lineno)
+enum selint_error insert_xperm_av_rule(struct policy_node **cur,
+                                       enum av_rule_flavor flavor,
+                                       struct string_list *sources,
+                                       struct string_list *targets,
+                                       struct string_list *object_classes,
+                                       const char *operation,
+                                       struct string_list *perms,
+                                       unsigned int lineno)
+{
+
+	struct xav_rule_data *xav_data = malloc(sizeof(struct xav_rule_data));
+
+	xav_data->flavor = flavor;
+	xav_data->sources = sources;
+	xav_data->targets = targets;
+	xav_data->object_classes = object_classes;
+	xav_data->operation = strdup(operation);
+	xav_data->perms = perms;
+
+	union node_data nd;
+	nd.xav_data = xav_data;
+
+	enum selint_error ret = insert_policy_node_next(*cur,
+	                                                NODE_XAV_RULE,
+	                                                nd,
+	                                                lineno);
+	if (ret != SELINT_SUCCESS) {
+		free_xav_rule_data(xav_data);
+		return ret;
+	}
+
+	*cur = (*cur)->next;
+
+	return SELINT_SUCCESS;
+}
+
+
+enum selint_error insert_role_allow(struct policy_node **cur,
+                                    struct string_list *from_roles,
+                                    struct string_list *to_roles, unsigned int lineno)
 {
 	struct role_allow_data *ra_data = malloc(sizeof(struct role_allow_data));
 
-	ra_data->from = strdup(from_role);
-	ra_data->to = strdup(to_role);
+	ra_data->from = from_roles;
+	ra_data->to = to_roles;
 
 	union node_data nd;
 	nd.ra_data = ra_data;
@@ -347,6 +383,7 @@ enum selint_error insert_type_transition(struct policy_node **cur,
 enum selint_error insert_role_transition(struct policy_node **cur,
                                          struct string_list *sources,
                                          struct string_list *targets,
+                                         struct string_list *object_classes,
                                          char *default_role,
                                          unsigned int lineno)
 {
@@ -355,6 +392,7 @@ enum selint_error insert_role_transition(struct policy_node **cur,
 
 	rt_data->sources = sources;
 	rt_data->targets = targets;
+	rt_data->object_classes = object_classes;
 	rt_data->default_role = strdup(default_role);
 
 	union node_data nd;
@@ -473,6 +511,24 @@ enum selint_error insert_semicolon(struct policy_node **cur, unsigned int lineno
 	return SELINT_SUCCESS;
 }
 
+enum selint_error insert_m4simplemacro(struct policy_node **cur, char *name, unsigned int lineno)
+{
+
+	union node_data nd;
+	nd.str = name;
+
+	enum selint_error ret = insert_policy_node_next(*cur,
+	                                                NODE_M4_SIMPLE_MACRO,
+	                                                nd,
+	                                                lineno);
+	if (ret != SELINT_SUCCESS) {
+		return ret;
+	}
+
+	*cur = (*cur)->next;
+	return SELINT_SUCCESS;
+}
+
 static enum selint_error begin_block(struct policy_node **cur,
                               enum node_flavor block_type, union node_data nd,
                               unsigned int lineno)
@@ -540,6 +596,20 @@ enum selint_error end_optional_else(struct policy_node **cur)
 	return end_block(cur, NODE_OPTIONAL_ELSE);
 }
 
+enum selint_error begin_boolean_policy(struct policy_node **cur,
+                                       unsigned int lineno)
+{
+	union node_data nd;
+	nd.str = NULL;
+	return begin_block(cur, NODE_BOOLEAN_POLICY, nd, lineno);
+}
+
+enum selint_error end_boolean_policy(struct policy_node **cur)
+{
+
+	return end_block(cur, NODE_BOOLEAN_POLICY);
+}
+
 enum selint_error begin_tunable_policy(struct policy_node **cur,
                                        unsigned int lineno)
 {
@@ -631,6 +701,30 @@ enum selint_error end_ifdef(struct policy_node **cur)
 	return end_block(cur, NODE_IFDEF);
 }
 
+enum selint_error begin_ifelse(struct policy_node **cur, unsigned int lineno)
+{
+	union node_data nd;
+	nd.str = NULL;
+	return begin_block(cur, NODE_IFELSE, nd, lineno);
+}
+
+enum selint_error end_ifelse(struct policy_node **cur)
+{
+	return end_block(cur, NODE_IFELSE);
+}
+
+enum selint_error begin_m4_argument(struct policy_node **cur, unsigned int lineno)
+{
+	union node_data nd;
+	nd.str = NULL;
+	return begin_block(cur, NODE_M4_ARG, nd, lineno);
+}
+
+enum selint_error end_m4_argument(struct policy_node **cur)
+{
+	return end_block(cur, NODE_M4_ARG);
+}
+
 enum selint_error save_command(struct policy_node *cur, const char *comm)
 {
 	if (comm == NULL || cur == NULL) {
@@ -688,7 +782,7 @@ static enum selint_error insert_attribute(struct policy_node **cur, enum attr_fl
 
 	if ((*cur)->parent &&
 	    (*cur)->parent->flavor == NODE_INTERFACE_DEF &&
-	    (check_transform_interface_suffix((*cur)->parent->data.str) ||
+	    (is_transform_interface((*cur)->parent->data.str) ||
 	     0 == strcmp(get_current_module_name(), "mls") ||
 	     0 == strcmp(get_current_module_name(), "mcs"))) {
 		mark_transform_if((*cur)->parent->data.str);
@@ -717,6 +811,4 @@ void cleanup_parsing()
 	free_permmacros();
 
 	free_all_maps();
-
-	yylex_destroy();
 }
