@@ -14,6 +14,10 @@
 * limitations under the License.
 */
 
+#include <ctype.h>
+#include <stdio.h>
+
+#include "color.h"
 #include "te_checks.h"
 #include "maps.h"
 #include "tree.h"
@@ -45,6 +49,9 @@ struct check_result *check_te_order(const struct check_data *data,
 		case ORDER_REF:
 			calculate_longest_increasing_subsequence(node, order_data, compare_nodes_refpolicy);
 			break;
+		case ORDER_LIGHT:
+			calculate_longest_increasing_subsequence(node, order_data, compare_nodes_refpolicy_light);
+			break;
 		case ORDER_LAX:
 			calculate_longest_increasing_subsequence(node, order_data, compare_nodes_refpolicy_lax);
 			break;
@@ -66,7 +73,11 @@ struct check_result *check_te_order(const struct check_data *data,
 				if (order_data->nodes[i].in_order) {
 					return NULL;
 				} else {
-					char *reason_str = get_ordering_reason(order_data, order_node_arr_index);
+					char *reason_str = get_ordering_reason(order_data, order_node_arr_index, data->config_check_data->order_conf);
+					if (!reason_str) {
+						return alloc_internal_error("Failed to compute reason C-001");
+					}
+
 					struct check_result *to_ret = make_check_result('C',
 					                                                C_ID_TE_ORDER,
 					                                                "%s",
@@ -89,6 +100,9 @@ struct check_result *check_unordered_perms(__attribute__((unused)) const struct
 	if (node->flavor == NODE_AV_RULE) {
 		cur = node->data.av_data->perms;
 		flavor = "av rule";
+	} else if (node->flavor == NODE_XAV_RULE) {
+		cur = node->data.xav_data->perms;
+		flavor = "xav rule";
 	} else if (node->flavor == NODE_DECL) {
 		// ignore non-class declarations
 		if (node->data.d_data->flavor != DECL_CLASS) {
@@ -101,7 +115,7 @@ struct check_result *check_unordered_perms(__attribute__((unused)) const struct
 	}
 
 	while (cur) {
-		if (prev && strcmp(prev->string, "~") != 0) {
+		if (prev && strcmp(prev->string, "~") != 0 && strcmp(cur->string, "-") != 0) {
 			const int compare = strcmp(prev->string, cur->string);
 
 			if (compare > 0) {
@@ -126,6 +140,36 @@ struct check_result *check_unordered_perms(__attribute__((unused)) const struct
 
 }
 
+struct check_result *check_no_self(__attribute__((unused)) const struct check_data *data,
+                                   const struct policy_node *node)
+{
+	if (node->flavor != NODE_AV_RULE && node->flavor != NODE_XAV_RULE) {
+		return alloc_internal_error("Bad node type given to check C-007");
+	}
+	struct av_rule_data *av_data = node->data.av_data;
+
+	if (av_data->sources->next ||
+	    av_data->targets->next ||
+	    0 == strcmp(av_data->targets->string, "self") ||
+	    0 != strcmp(av_data->sources->string, av_data->targets->string)) {
+		return NULL;
+	}
+
+	if (av_data->sources->string[0] == '$') {
+		// On variables, we skip unless they are "_t" suffixed
+		if (!ends_with(av_data->sources->string, strlen(av_data->sources->string), "_t", 2)) {
+			return NULL;
+		}
+	} else if (!look_up_in_decl_map(av_data->sources->string, DECL_TYPE)) {
+		// skip attributes
+		return NULL;
+	}
+
+	return make_check_result('C', C_ID_SELF,
+	                         "Recommend use of self keyword instead of redundant type");
+
+}
+
 struct check_result *check_require_block(const struct check_data *data,
                                          const struct policy_node *node)
 {
@@ -168,6 +212,30 @@ struct check_result *check_bare_module_statement(__attribute__((unused)) const s
 	return NULL;
 }
 
+// check if all classes are netlink socket classes
+static bool all_netlink_socket_classes(const struct string_list *classes)
+{
+	for (; classes; classes = classes->next) {
+		if (0 != strncmp(classes->string, "netlink_", strlen("netlink_"))) {
+			return false;
+		}
+	}
+
+	return true;
+}
+
+// check if all classes are socket classes
+static bool all_socket_classes(const struct string_list *classes)
+{
+	for (; classes; classes = classes->next) {
+		if (!ends_with(classes->string, strlen(classes->string), "_socket", strlen("_socket"))) {
+			return false;
+		}
+	}
+
+	return true;
+}
+
 // check if '$STR' ends with '$SUFFIX_perms'
 static bool ends_with_suffix_perms(const char *str, size_t str_len, const char *suffix, size_t suffix_len)
 {
@@ -177,17 +245,38 @@ static bool ends_with_suffix_perms(const char *str, size_t str_len, const char *
 
 	// no need to check last 6 characters are actual '_perms'
 	// we call this only on strings we have checked to have this suffix
-	return (0 == strncmp(str + str_len - (suffix_len + strlen("_perms")), suffix, suffix_len));
+	return (0 == strncmp(str + str_len - (suffix_len + strlen("_perms")),
+			     suffix,
+			     suffix_len));
+}
+static bool ends_with_all_suffix_perms(const char *str, size_t str_len, const struct string_list *classes)
+{
+	for (; classes; classes = classes->next) {
+		if (!ends_with_suffix_perms(str, str_len, classes->string, strlen(classes->string))) {
+			// check class alias as fallback
+			static const char *const class_aliases[][2] = {
+				{ "chr_file", "term"   },
+				{ "process",  "signal" },
+			};
+			const char *class_alias = NULL;
+			for (size_t i = 0; i < (sizeof class_aliases / sizeof *class_aliases); ++i) {
+				if (0 == strcmp(classes->string, class_aliases[i][0])) {
+					class_alias = class_aliases[i][1];
+					break;
+				}
+			}
+			if (!class_alias || !ends_with_suffix_perms(str, str_len, class_alias, strlen(class_alias))) {
+				return false;
+			}
+		}
+	}
+
+	return true;
 }
 
 struct check_result *check_perm_macro_class_mismatch(__attribute__((unused)) const struct check_data *data,
                                                      const struct policy_node *node)
 {
-	static const char *const class_aliases[][2] = {
-		{ "chr_file", "term"   },
-		{ "process",  "signal" },
-	};
-
 	static const char *const file_suffix_classes[] = {
 		"lnk_file",
 		"chr_file",
@@ -196,26 +285,16 @@ struct check_result *check_perm_macro_class_mismatch(__attribute__((unused)) con
 		"fifo_file",
 	};
 
-	const char *class_name = node->data.av_data->object_classes->string;
-	const size_t class_name_len = strlen(class_name);
+	const struct string_list *classes = node->data.av_data->object_classes;
 
-	// ignore multi class av rules
-	if (node->data.av_data->object_classes->next ||
-	    ends_with(class_name, class_name_len, "_class_set", strlen("_class_set"))) {
+	// ignore class set av rules
+	if (ends_with(classes->string, strlen(classes->string), "_class_set", strlen("_class_set"))) {
 		return NULL;
 	}
 
-	const char *class_alias = NULL;
-	for (size_t i = 0; i < (sizeof class_aliases / sizeof *class_aliases); ++i) {
-		if (0 == strcmp(class_name, class_aliases[i][0])) {
-			class_alias = class_aliases[i][1];
-			break;
-		}
-	}
-	const size_t class_alias_len = class_alias ? strlen(class_alias) : 0;
-	const bool is_file_class = (0 == strcmp(class_name, "file"));
-	const bool is_netlink_socket_class = (0 == strncmp(class_name, "netlink_", strlen("netlink_")));
-	const bool is_socket_class = ends_with(class_name, class_name_len, "_socket", strlen("_socket"));
+	const bool is_file_class = str_in_sl("file", classes);
+	const bool is_netlink_socket_class = all_netlink_socket_classes(classes);
+	const bool is_socket_class = all_socket_classes(classes);
 
 	for (const struct string_list *perms = node->data.av_data->perms; perms; perms = perms->next) {
 		const size_t perm_len = strlen(perms->string);
@@ -226,7 +305,8 @@ struct check_result *check_perm_macro_class_mismatch(__attribute__((unused)) con
 		}
 
 		// ignore permissions matching 'something[_something]_$CLASSNAME_perms'
-		if (ends_with_suffix_perms(perms->string, perm_len, class_name, class_name_len)) {
+		// and 'something[_something]_$CLASSNAMEALIAS_perms'
+		if (ends_with_all_suffix_perms(perms->string, perm_len, classes)) {
 			// report usage of macros matching different class with actual class as suffix
 			// e.g. report 'something_fifo_file_perms' for class 'file'
 			if (is_file_class) {
@@ -253,16 +333,11 @@ struct check_result *check_perm_macro_class_mismatch(__attribute__((unused)) con
 			continue;
 		}
 
-		// ignore permissions 'something[_something]_$CLASSNAMEALIAS_perms'
-		if (class_alias && ends_with_suffix_perms(perms->string, perm_len, class_alias, class_alias_len)) {
-			continue;
-		}
-
 report:
 		return make_check_result('S', S_ID_PERM_SUFFIX,
 					 "Permission macro %s does not match class %s",
 					 perms->string,
-					 class_name);
+					 classes->next ? "(multi class av rule)" : classes->string);
 	}
 
 	return NULL;
@@ -290,8 +365,8 @@ struct check_result *check_perm_macro_available(__attribute__((unused)) const st
 	}
 
 	struct check_result *res = make_check_result('S', S_ID_PERMMACRO,
-						"%s",
-						check_str);
+						     "%s",
+						     check_str);
 	free(check_str);
 	return res;
 }
@@ -604,30 +679,43 @@ struct check_result *check_declaration_interface_nameclash(__attribute__((unused
 	return NULL;
 }
 
-struct check_result *check_unknown_permission_macro(__attribute__((unused)) const struct check_data
-						    *data,
-						    const struct policy_node
-						    *node)
+bool check_unknown_permission_condition()
 {
-	static unsigned int permmacros_count = (unsigned int)-1;
-
-	if (permmacros_count == (unsigned int)-1) {
-		permmacros_count = permmacros_map_count();
+	// ignore if no permission or permission macro have been parsed
+	if (permmacros_map_count() == 0) {
+		printf("%sNote%s: Check E-007 is not performed because no permission macro has been parsed.\n",
+		       color_note(), color_reset());
+		return false;
+	}
+	if (decl_map_count(DECL_PERM) == 0) {
+		printf("%sNote%s: Check E-007 is not performed because no permission has been parsed.\n",
+		       color_note(), color_reset());
+		return false;
 	}
 
-	// ignore if no permission macro was parsed
-	if (permmacros_count == 0) {
-		return NULL;
-	}
+	return true;
+}
 
+struct check_result *check_unknown_permission(__attribute__((unused)) const struct check_data
+					      *data,
+					      const struct policy_node
+					      *node)
+{
 	for (const struct string_list *cur = node->data.av_data->perms; cur; cur = cur->next) {
-		// ignore permissions without '_perms' suffix; they are probably not macros
-		if (!ends_with(cur->string, strlen(cur->string), "_perms", strlen("_perms"))) {
+
+		if (0 == strcmp(cur->string, "*") ||
+		    0 == strcmp(cur->string, "~")) {
 			continue;
 		}
 
 		// ignore generated all_ permission macros
-		if (0 == strncmp(cur->string, "all_", strlen("all_"))) {
+		if (0 == strncmp(cur->string, "all_", strlen("all_")) &&
+		    ends_with(cur->string, strlen(cur->string), "_perms", strlen("_perms"))) {
+			continue;
+		}
+
+		if (look_up_in_decl_map(cur->string, DECL_PERM)) {
+			// TODO: check if class supports this permission
 			continue;
 		}
 
@@ -635,10 +723,128 @@ struct check_result *check_unknown_permission_macro(__attribute__((unused)) cons
 			continue;
 		}
 
-		return make_check_result('E', E_ID_UNKNOWN_PERMMACRO,
-					 "Unknown permission macro %s used",
+		return make_check_result('E', E_ID_UNKNOWN_PERM,
+					 "Unknown permission %s used",
 					 cur->string);
 	}
 
 	return NULL;
 }
+
+bool check_unknown_class_condition()
+{
+	// ignore if no class has been parsed
+	if (decl_map_count(DECL_CLASS) == 0) {
+		printf("%sNote%s: Check E-008 is not performed because no class has been parsed.\n",
+		       color_note(), color_reset());
+		return false;
+	}
+
+	return true;
+}
+
+struct check_result *check_unknown_class(__attribute__((unused)) const struct check_data
+					 *data,
+					 const struct policy_node
+					 *node)
+{
+	const struct string_list *object_classes;
+	switch (node->flavor) {
+	case NODE_AV_RULE:
+		object_classes = node->data.av_data->object_classes;
+		break;
+	case NODE_RT_RULE:
+		object_classes = node->data.rt_data->object_classes;
+		break;
+	case NODE_TT_RULE:
+		object_classes = node->data.tt_data->object_classes;
+		break;
+	default:
+		return alloc_internal_error("Invalid node type for `check_unknown_class`");
+	}
+
+	for (const struct string_list *cur = object_classes; cur; cur = cur->next) {
+		// ignore interface parameters
+		if (cur->string[0] == '$' && isdigit((unsigned char)cur->string[1])) {
+			continue;
+		}
+
+		// ignore class sets
+		if (ends_with(cur->string, strlen(cur->string), "_class_set", strlen("_class_set"))) {
+			continue;
+		}
+
+		if (look_up_in_decl_map(cur->string, DECL_CLASS)) {
+			continue;
+		}
+
+		return make_check_result('E', E_ID_UNKNOWN_CLASS,
+					 "Unknown class %s used",
+					 cur->string);
+	}
+
+	return NULL;
+}
+
+struct check_result *check_empty_block(__attribute__((unused)) const struct check_data
+                                       *data,
+                                       const struct policy_node
+                                       *node)
+{
+	for (const struct policy_node *cur = node->first_child; cur; cur = cur->next) {
+		if (cur->flavor == NODE_START_BLOCK ||
+		    cur->flavor == NODE_COMMENT ||
+		    cur->flavor == NODE_SEMICOLON) {
+			continue;
+		}
+
+		// found a statement
+		return NULL;
+	}
+
+	return make_check_result('E', E_ID_EMPTY_BLOCK,
+				 "Empty block found");
+}
+
+static const struct policy_node *is_ifelse_argument(const struct policy_node *cur)
+{
+	for(; cur && cur->parent; cur = cur->parent) {
+		if (cur->flavor == NODE_M4_ARG && cur->parent->flavor == NODE_IFELSE) {
+			return cur;
+		}
+	}
+
+	return NULL;
+}
+
+struct check_result *check_stray_word(const struct check_data
+				      *data,
+				      const struct policy_node
+				      *node)
+{
+	const char *macro_name = node->data.str;
+
+	if (str_in_sl(macro_name, data->config_check_data->custom_te_simple_macros)) {
+		return NULL;
+	}
+
+	// ignore comparison arguments to ifelse
+	// (do not ignore last node of block, which is never a comparison argument)
+	const struct policy_node *ifelse_arument = is_ifelse_argument(node);
+	if (ifelse_arument && ifelse_arument->next) {
+		int position = 0;
+		for (const struct policy_node *cur = ifelse_arument; cur->prev; cur = cur->prev) {
+			position++;
+		}
+
+		position = position % 3;
+
+		if ((position == 1 || position == 2)) {
+			return NULL;
+		}
+	}
+
+	return make_check_result('E', E_ID_STRAY_WORD,
+				 "Found stray word %s. If it is a simple m4 macro please add an selint-disable comment or ignore in the SELint configuration file.",
+				 macro_name);
+}
