@@ -159,6 +159,9 @@ struct string_list *get_names_in_node(const struct policy_node *node)
 
 	switch (node->flavor) {
 	case NODE_AV_RULE:
+	case NODE_XAV_RULE:
+		// Since the common elements are ordered identically, we can just look
+		// at the common subset for the XAV rule
 		av_data = node->data.av_data;
 		cur = ret = copy_string_list(av_data->sources);
 		if (cur) {
@@ -230,7 +233,11 @@ struct string_list *get_names_in_node(const struct policy_node *node)
 
 	case NODE_ROLE_ALLOW:
 		ra_data = node->data.ra_data;
-		ret = sl_from_strs(2, ra_data->from, ra_data->to);
+		cur = ret = copy_string_list(ra_data->from);
+		while (cur->next) {
+			cur = cur->next;
+		}
+		cur->next = copy_string_list(ra_data->to);
 		break;
 
 	case NODE_ROLE_TYPES:
@@ -257,12 +264,16 @@ struct string_list *get_names_in_node(const struct policy_node *node)
 	   NODE_IF_FILE,
 	   NODE_FC_FILE,
 	   NODE_SPT_FILE,
+	   NODE_AV_FILE,
 	   NODE_HEADER,
 	   NODE_M4_CALL,
+	   NODE_M4_SIMPLE_MACRO,
 	   NODE_OPTIONAL_POLICY,
 	   NODE_OPTIONAL_ELSE,
+	   NODE_BOOLEAN_POLICY,
 	   NODE_TUNABLE_POLICY,
 	   NODE_IFDEF,
+	   NODE_IFELSE,
 	   NODE_M4_ARG,
 	   NODE_START_BLOCK,
 	   NODE_INTERFACE_DEF,
@@ -360,6 +371,9 @@ enum selint_error free_policy_node(struct policy_node *to_free)
 	case NODE_AV_RULE:
 		free_av_rule_data(to_free->data.av_data);
 		break;
+	case NODE_XAV_RULE:
+		free_xav_rule_data(to_free->data.xav_data);
+		break;
 	case NODE_ROLE_ALLOW:
 		free_ra_data(to_free->data.ra_data);
 		break;
@@ -437,8 +451,23 @@ enum selint_error free_av_rule_data(struct av_rule_data *to_free)
 	free_string_list(to_free->object_classes);
 	free_string_list(to_free->perms);
 
-	to_free->sources = to_free->targets = to_free->object_classes =
-		to_free->perms = NULL;
+	free(to_free);
+
+	return SELINT_SUCCESS;
+}
+
+enum selint_error free_xav_rule_data(struct xav_rule_data *to_free)
+{
+
+	if (to_free == NULL) {
+		return SELINT_BAD_ARG;
+	}
+
+	free_string_list(to_free->sources);
+	free_string_list(to_free->targets);
+	free_string_list(to_free->object_classes);
+	free(to_free->operation);
+	free_string_list(to_free->perms);
 
 	free(to_free);
 
@@ -452,8 +481,8 @@ enum selint_error free_ra_data(struct role_allow_data *to_free)
 		return SELINT_BAD_ARG;
 	}
 
-	free(to_free->from);
-	free(to_free->to);
+	free_string_list(to_free->from);
+	free_string_list(to_free->to);
 	free(to_free);
 
 	return SELINT_SUCCESS;
@@ -501,6 +530,7 @@ enum selint_error free_role_transition_data(struct role_transition_data
 
 	free_string_list(to_free->sources);
 	free_string_list(to_free->targets);
+	free_string_list(to_free->object_classes);
 	free(to_free->default_role);
 
 	free(to_free);
